/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.deploytool;

/**
 * This is the CCM XML partitionner operator.
 *
 * @author <a href="mailto:Christophe.Contreras@lifl.fr">Christophe Contreras</a>
 *
 * @version 0.1
 */

public class Partition
     extends java.util.Hashtable
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the calling process. */
    private DeploymentProcess _deployer;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    Partition (DeploymentProcess deployer)
    {
        // attach the calling deployment process
        this._deployer = deployer;

        _deployer.getConsole().message("Initiating the partition...");

        // TODO: parse XML and prepare the cache

        _deployer.getConsole().message("Partition is ready");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Prepares an information table from a homeplacmeent descriptor
     *
     * @param homePlacement The Java version of the descriptor
     * @param destination The predefined destination in collocations
     *
     * @return the table
     */
    private java.util.Hashtable
    gatherHomeInfo (
        org.objectweb.ccm.descriptor.componentassembly.HomeplacementBean homePlacement
        , String destination
        )
    {
        // the table that will be fed and returned
        java.util.Hashtable homeInfos
            = new java.util.Hashtable();

        // first store the ID of this homeplacement
        homeInfos.put(
            "id"
            ,homePlacement.getId()
            );
	
        // check for a predefined destination
        if (destination != null)
        {
            homeInfos.put(
                "destination"
                , destination
                );
        }else{
            if (homePlacement.getDestination().getValue() != null)
            {
                homeInfos.put(
                    "destination"
                    ,homePlacement
                        .getDestination()
                        .getValue()
                    );
            }else{
                _deployer
                    .getConsole().error(
                        "No <destination> found, displaying erroneous XML..."
                        );
                try {
                    homePlacement
                        .marshal(   
                            _deployer
                                .getConsole().getErrorStream()
                            );
                } catch (java.io.IOException io_exception) {
                    _deployer
                        .getConsole().error(
                            "XML description couldn't be displayed during this error reporting"
                            );
                }
            }
        }

        // look for the softpkg descriptor for this home from the assembly
        java.util.List filesList
            = _deployer
                .getAssemblyObject()
                .getComponentfiles()
                .getComponentfileList();

        for (java.util.Iterator i = filesList.iterator() ; i.hasNext() ; ) {
	
            org.objectweb.ccm.descriptor.componentassembly.ComponentfileBean componentFile
                = (org.objectweb.ccm.descriptor.componentassembly.ComponentfileBean) i.next();

            // compare the ids from home and componentfile declarations
            if ( homePlacement.getComponentfileref().getIdref().equals (componentFile.getId()) )
            {
                // this is the CSD we need
                java.io.File softpkgFile
                    = new java.io.File (
                        org.objectweb.ccm.deploytool.DeploymentProcess
                            .getBaseDir()
                        + componentFile
                            .getFileinarchive()
                            .getName()
                        );

                // open the softpkg descriptor
                org.objectweb.ccm.descriptor.softpkg.SoftpkgBean softpkgObject = null;
                try {
                    softpkgObject
                        = org.objectweb.ccm.descriptor.softpkg.SoftpkgImplBean
                            .unmarshalBean(softpkgFile);
                } catch (java.io.IOException exception) {
                    _deployer.getConsole().error(
                        "IO error reading defined softpkg file, displaying erroneous XML..."
                        );
                    try {
                        componentFile
                            .marshal(   
                                _deployer
                                    .getConsole().getErrorStream()
                                );
                    } catch (java.io.IOException io_exception) {
                        _deployer
                            .getConsole().error(
                                "XML description couldn't be displayed during this error reporting"
                                );
                    }
                }
		
                // retrieve the first implementation from the list
                org.objectweb.ccm.descriptor.softpkg.ImplementationBean implemToInstall
                    = (org.objectweb.ccm.descriptor.softpkg.ImplementationBean)
                        softpkgObject
                            .getImplementationList()
                            .get(0);

                // feed the home info table with the implem ID
                homeInfos.put(
                    "implemId"
                    ,implemToInstall.getId()
                    );

                // feed the home info table with the archive filename
                homeInfos.put(
                    "archiveFile"
                    ,( (org.objectweb.ccm.descriptor.softpkg.CodeBean)
                        implemToInstall
                            .getCodeList()
                            .get(0)
                        ).getFileinarchive().getName()
                    );

                // feed the home info table with the entry point
                homeInfos.put(
                    "entryPoint"
                    ,( (org.objectweb.ccm.descriptor.softpkg.CodeBean)
                	implemToInstall
                            .getCodeList()
                            .get(0)
                        ).getEntrypoint().getValue()
                    );
            }
        }

        // return the table to the calling process
        return (homeInfos);
    }

    /**
     * Prepares an information table from a component instance descriptor
     *
     * @param instanceToSet The descriptor of the instance
     *
     * @return the table
     */
    private java.util.Hashtable
    gatherComponentInstanceInfo (
        org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationBean instanceToSet
        )
    {
        // this table will be fed then returned to the calling processor
        java.util.Hashtable instanceInfos
            = new java.util.Hashtable();

        // store an identification for the instance
        instanceInfos.put(
            "instanceId"
            , instanceToSet.getId()
            );

        // this list will contain every property to set on the instance
        java.util.List storedPropertiesList
            = new java.util.LinkedList();

        if ( instanceToSet.getComponentproperties() != null )
        {
            // open the property file for the instance if defined
            java.io.File propertiesFile
                = new java.io.File (
                        org.objectweb.ccm.deploytool.DeploymentProcess
                            .getBaseDir()
                        + instanceToSet
                            .getComponentproperties()
                            .getFileinarchive()
                            .getName()
                        );

            // retrieving the Java version of the properties descriptor
            org.objectweb.ccm.descriptor.properties.PropertiesBean propertiesObject = null;
            try {
                propertiesObject
                    = org.objectweb.ccm.descriptor.properties.PropertiesImplBean
                        .unmarshalBean(propertiesFile);
            } catch (java.io.IOException exception) {
                _deployer
                    .getConsole().error(
                        "IO error on defined property file, displaying erroneous XML..."
                        );
                    try {
                        instanceToSet
                            .marshal(
                                _deployer
                                    .getConsole().getErrorStream()
                                );
                    } catch (java.io.IOException io_exception) {
                        _deployer
                            .getConsole().error(
                                "XML description couldn't be displayed during this error reporting"
                                );
                    }
            }

            // get the list of simple properties
            java.util.List propertiesList
                = propertiesObject
                    .getSimpleList();
	    
            for (java.util.Iterator i = propertiesList.iterator() ; i.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.properties.SimpleBean property
                    = (org.objectweb.ccm.descriptor.properties.SimpleBean) i.next();
		
                // build an array with name, type, value
                String[] name_type_value
                    = {
                        property.getName()
                        , property.getType()
                        , property.getValue().getValue()
                        };

                // add it to the properties list
                storedPropertiesList
                    .add(name_type_value);
            }
	    
        }
	
        // add the properties list to the table
        instanceInfos.put(
            "propertiesList"
            , storedPropertiesList
            );

        // return the table to the calling process	
        return (instanceInfos);
    }

    /**
     * Go through a list of home placements and deploy them
     * for each home placement, instantiates the wanted components
     *
     * @param placementsList The list to go through.
     * @param destination The predefined destination in collocations
     *
     * @return a boolean indicating success
     */
    private boolean
    processHomeplacementList ( java.util.List placementsList, String destination )
    {
        for (java.util.Iterator i = placementsList.iterator() ; i.hasNext() ; )
        {
            org.objectweb.ccm.descriptor.componentassembly.HomeplacementBean homePlacement
                = (org.objectweb.ccm.descriptor.componentassembly.HomeplacementBean) i.next();

            // deploying the home with success test
            if ( !
                deploy_home(
                    gatherHomeInfo(
                        homePlacement
                        , destination
                        )
                    )
                )
                return false;

            // retrieving the reference
            org.omg.Components.CCMHome homeDeployed
                = (org.omg.Components.CCMHome)
                    this.get(
                        homePlacement.getId()
                        );

            // storing the reference in the DeployInfo extension of the descriptor
            ( (org.objectweb.ccm.descriptor.componentassembly.HomeplacementDeployInfo)
                homePlacement
                    .getExtensionManager()
                    .getExtensionByName("DeployInfo")
                    .getInstance() 
                ).setCorbaReference(homeDeployed);

            // check the descriptor for defined NS registrations
            java.util.List ns_registrations
                = homePlacement
                    .getRegisterwithnamingList();
            for (java.util.Iterator ns_reg_iterator = ns_registrations.iterator() ; ns_reg_iterator.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.RegisterwithnamingBean register_with_naming
                    = (org.objectweb.ccm.descriptor.componentassembly.RegisterwithnamingBean) ns_reg_iterator.next();
                
                _deployer
                    .getConsole().message(
                        "Binding the "
                        + register_with_naming.getName()
                        + " homeplacement to the NS"
                        );

                // proceed with binding
                org.objectweb.ccm.CORBA.TheNameService
                    .getNamingContext()
                    .rebind(
                        register_with_naming.getName()
                        , (org.omg.CORBA.Object) homeDeployed
                        );
            }
            // components instantiations
            java.util.List instantiationsList = homePlacement.getComponentinstantiationList();
            for (java.util.Iterator comp_iterator = instantiationsList.iterator() ; comp_iterator.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationBean instanceToSet
                    = (org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationBean) comp_iterator.next();

                // call the deployment and setup for the described component instantiation
                if ( !
                    deploy_and_set_component(
                        gatherComponentInstanceInfo(instanceToSet)
                        , homeDeployed
                        )
                    )
                    return false;

                // retrieve the stored reference
                org.omg.Components.CCMObject objectDeployed
                    = (org.omg.Components.CCMObject)
                        this.get(
                            instanceToSet.getId()
                            );

                // storing the reference in the DeployInfo extension of the descriptor
                ( (org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationDeployInfo)
                    instanceToSet
                        .getExtensionManager()
                        .getExtensionByName("DeployInfo")
                        .getInstance()
                    ).setCorbaReference(objectDeployed);

                // check the descriptor for defined NS registrations
                ns_registrations
                    = instanceToSet
                        .getRegistercomponentList();
                
                for (java.util.Iterator comp_reg_iterator = ns_registrations.iterator() ; comp_reg_iterator.hasNext() ; )
                {
                    org.objectweb.ccm.descriptor.componentassembly.RegistercomponentBean comp_registration
                        = (org.objectweb.ccm.descriptor.componentassembly.RegistercomponentBean) comp_reg_iterator.next();

                    // get NS registration tag
                    java.util.List comp_ns_registrations
                        = comp_registration
                            .getRegisterwithnamingList();

                    for (java.util.Iterator ns_reg_iterator = comp_ns_registrations.iterator() ; ns_reg_iterator.hasNext() ; )
                    {
                        org.objectweb.ccm.descriptor.componentassembly.RegisterwithnamingBean register_with_naming
                            = (org.objectweb.ccm.descriptor.componentassembly.RegisterwithnamingBean) ns_reg_iterator.next();

                        _deployer
                            .getConsole().message(
                                "Binding the "
                                + register_with_naming.getName()
                                + " componentinstance to the NS"
                                );

                        // proceed with binding
                        org.objectweb.ccm.CORBA.TheNameService
                            .getNamingContext()
                            .rebind(
                                register_with_naming.getName()
                                , (org.omg.CORBA.Object) objectDeployed
                                );
                    }
                }

                // notify the connector that a new compoennt instance is ready
                if ( !
                    _deployer
                        .getConnector()
                        .componentReady(
                            instanceToSet.getId()
                            )
                    )
                    return false;
            }
        }

        // everything was OK during homePlacement list treatment
        return true;
    }

    /**
     * Deployment of a home
     *
     * @param homeInfos The home's informations hashtable
     *
     * @return a boolean indicating success
     */
    private boolean
    deploy_home (java.util.Hashtable homeInfos)
    {
        // Obtain the component servers.
        org.omg.CosNaming.NameComponent[] ncomp
            = new org.omg.CosNaming.NameComponent[1];

        ncomp[0]
            = new org.omg.CosNaming.NameComponent(
                (String)homeInfos
                    .get("destination")
                ,""
                );

        org.omg.CORBA.Object obj = null;
        try {
            obj
                = org.objectweb.ccm.CORBA.TheNameService
                    .getNamingContext()
                    .getNamingContext()
                    .resolve(ncomp);
        } catch (org.omg.CosNaming.NamingContextPackage.NotFound exception) {
            _deployer.getConsole().error(
                "The NS could not resolve a \""
                + (String)homeInfos.get("destination")
                + "\" destination"
                );
            return false;
        } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed exception) {
            _deployer.getConsole().error(
                "The NS could not resolve a \""
                + (String)homeInfos.get("destination")
                + "\" destination"
                );
            return false;
        } catch (org.omg.CosNaming.NamingContextPackage.InvalidName exception) {
            _deployer.getConsole().error(
                "Invalid name error during NS resolve of the \""
                + (String)homeInfos.get("destination")
                + "\" destination for the \""
                + (String)homeInfos.get("implemId")
                + "\" home"
                );
            return false;
        }

        org.objectweb.ccm.Deployment.Server server
            = org.objectweb.ccm.Deployment.ServerHelper
                .narrow(obj);

        // Obtain the home factories and archive installators.
        org.omg.Components.Deployment.ComponentServer server_cs
            = server
                .provide_component_server();
        org.omg.Components.Deployment.ComponentInstallation server_inst
            = server
                .provide_install();
		
        // Install archives.
    	_deployer.getConsole().message(
            "Installing archive "
            + homeInfos
                .get("archiveFile")
            + "...");
    	
        try {
            server_inst.install(
                (String)homeInfos
                    .get("implemId")
                , (String)homeInfos
                    .get("archiveFile")
                );
        } catch (org.omg.Components.Deployment.InvalidLocation exception) {
            _deployer.getConsole().error(
                "Cannot find installer server for the \""
                + (String)homeInfos.get("implemId")
                + "\" home"
                );
            return false;
        } catch (org.omg.Components.Deployment.InstallationFailure exception) {
            _deployer.getConsole().error(
                "The server failed installing the \""
                + (String)homeInfos.get("archiveFile")
                + "\" archive for the \""
                + (String)homeInfos.get("implemId")
                + "\" home"
                );
            return false;
        }
        
        // Install some plugins already provided by OpenCCM
        try {
            server_inst.install(
                "openccm_plugins"
                , "./OpenCCM_Plugins.jar"
                );
        } catch (org.omg.Components.Deployment.InvalidLocation exception) {
            _deployer
                .getConsole().error(
                    "Invalid location for server"
                    );
            return false;
        } catch (org.omg.Components.Deployment.InstallationFailure exception) {
            _deployer
                .getConsole().error(
                    "The server failed installing the archive"
                    );
            return false;
        }

    	org.omg.Components.ConfigValue[] config
            = new org.omg.Components.ConfigValue[0];

    	org.omg.Components.Deployment.Container server_cont = null;
        try {
            server_cont
                = server_cs
                    .create_container(config);
        } catch (org.omg.Components.CreateFailure exception) {
            _deployer
                .getConsole().error(
                    "Creation failure for container"
                    );
            return false;
        } catch (org.omg.Components.Deployment.InvalidConfiguration exception) {
            _deployer
                .getConsole().error(
                    "Container configuration was invalid"
                    );
            return false;
        }

    	// Create homes.

    	// install homes with this container configuration.
    	org.omg.Components.CCMHome homePlaced = null;
        try {
            homePlaced
                = server_cont
                    .install_home(
                        (String)homeInfos
                            .get("implemId")
                        , (String)homeInfos
                            .get("entryPoint")
                        , config
                        );
        } catch (org.omg.Components.Deployment.UnknownImplId exception) {
            _deployer
                .getConsole().error(
                    "Home ID was unknown"
                    );
            return false;
        } catch (org.omg.Components.Deployment.ImplEntryPointNotFound exception) {
            _deployer
                .getConsole().error(
                    "Could not find the entrypoint"
                    );
            return false;
        } catch (org.omg.Components.Deployment.InstallationFailure exception) {
            _deployer
                .getConsole().error(
                    "Failure during home installation"
                    );
            return false;
        } catch (org.omg.Components.Deployment.InvalidConfiguration exception) {
            _deployer
                .getConsole().error(
                    "Home configuration was invalid"
                    );
            return false;
        }

        _deployer.getConsole().message(
            "Successfully deployed home "
            + (String)homeInfos.get("implemId")
            );

        // store the reference in the partition
        this.put(
            (String)homeInfos.get("id")
            ,homePlaced
            );

        // everything was OK during the home deployment
        return (true);
    }

    /**
     * Deployment of a component and 
     *   setting of eventual properties
     *
     * @param instanceInfos table containing ID, properties
     *
     * @return a boolean indicating success
     */
    private boolean
    deploy_and_set_component(
        java.util.Hashtable instanceInfos
        , org.omg.Components.CCMHome homeDeployed)
    {
        // makes the home create a new component
        org.omg.Components.CCMObject new_component = null;
        try {
            new_component
                = org.omg.Components.KeylessCCMHomeHelper
                    .narrow(homeDeployed)
                        .create_component();
        } catch (org.omg.Components.CreateFailure exception) {
            _deployer.getConsole().error("Component creation failed");
            return false;
        }

        // go through the property list.
        for (java.util.Iterator i = ((java.util.List)instanceInfos.get("propertiesList")).iterator() ; i.hasNext() ; )
        {
            String[] name_type_value = (String[]) i.next();

            // build a set request
            org.omg.CORBA.Request request
                = new_component
                    ._request(
                        "_set_"
                        + name_type_value[0]
                        );
            // in case of a string type property
            if (name_type_value[1].equals( "string"))
            {
                request.add_in_arg().insert_string(name_type_value[2]);
            }

            // invocation of the set request
            request.invoke();
        }
	
        // registering the instance to the partition for future use
        this.put(
            (String) instanceInfos
                .get("instanceId")
            ,new_component
            );

        // everything was OK during component deployment and setting
        return true;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Operationnal method called by the deployment process
     * 1 - Get the partitioning informations from the assembly,
     * 2 - Process homeplacements, processcollocations, and hostcollocations
     *
     * @return a boolean indicating success
     */    
    public boolean
    operate()
    {
        // retrieve the assembly describing the partitioning
        org.objectweb.ccm.descriptor.componentassembly.ComponentassemblyBean assemblyObject
            = _deployer
                .getAssemblyObject();

        // destinations of root homeplacements are not defined yet
        String destination = null;

        // getting the list of described homeplacements
        java.util.List placementsList 
            = assemblyObject
                .getPartitioning()
                .getHomeplacementList();

        _deployer
            .getConsole().message(
                "Partition starts non collocated <homeplacement> list treatment"
                );

        // treatment of these root homeplacements
        if ( !
            processHomeplacementList(
                placementsList
                , destination
                )
            )
            return false;

        // getting the list of described process collocations
        java.util.List collocationList
            = assemblyObject
                .getPartitioning()
                .getProcesscollocationList();

        for (java.util.Iterator i = collocationList.iterator() ; i.hasNext() ; )
        {
            org.objectweb.ccm.descriptor.componentassembly.Processcollocation collocation
                = (org.objectweb.ccm.descriptor.componentassembly.Processcollocation) i.next();

            _deployer
                .getConsole().message(
                    "Partition starts <processcollocation> treatment"
                    );
            
            // checking a described destination for this collocation
            if (collocation.getDestination() != null)
            {
                destination
                    = collocation
                        .getDestination()
                        .getValue();
            }else{
                _deployer
                    .getConsole().error(
                        "No <destination> found, displaying erroneous XML..."
                        );
                try {
                    collocation
                        .marshal(   
                            _deployer
                                .getConsole().getErrorStream()
                            );
                } catch (java.io.IOException io_exception) {
                    _deployer
                        .getConsole().error(
                            "XML description couldn't be displayed during this error reporting"
                            );
                }
            }

            // treatment of the homeplacements in this collocation
            if ( !
                processHomeplacementList(
                    collocation
                        .getHomeplacementList()
                    , destination
                    )
                )
                return false;
        }
        
        // getting the list of described host collocations
        collocationList 
            = assemblyObject
                .getPartitioning()
                .getHostcollocationList();

        for (java.util.Iterator i = collocationList.iterator() ; i.hasNext() ; )
        {
            org.objectweb.ccm.descriptor.componentassembly.Hostcollocation collocation
                = (org.objectweb.ccm.descriptor.componentassembly.Hostcollocation) i.next();

            _deployer
                .getConsole().message(
                    "Partition starts <hostcollocation> treatment"
                    );

            // checking a described destination for this collocation
            if (collocation.getDestination() != null)
            {
                destination
                    = collocation
                        .getDestination()
                        .getValue();
            }else{
                _deployer
                    .getConsole().error(
                        "No <destination> found, displaying erroneous XML..."
                        );
                try {
                    collocation
                        .marshal(   
                            _deployer
                                .getConsole().getErrorStream()
                            );
                } catch (java.io.IOException io_exception) {
                    _deployer
                        .getConsole().error(
                            "XML description couldn't be displayed during this error reporting"
                            );
                }
            }

            // treatment of the homeplacements in this collocation
            if ( !
                processHomeplacementList(
                    collocation
                        .getHomeplacementList()
                    , destination
                    )
                )
                return false;
        }

        // everything was OK during the operations
        return true;
    }
}
