/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
import org.objectweb.corba.command.api.ApplicationServer;
import org.objectweb.util.cmdline.api.CommandLine;
import org.omg.PortableServer.POA;

// TODO: TheORB class must be moved in package org.objectweb.corba.util.lib
import org.objectweb.ccm.CORBA.TheORB;
// TODO: TheRootPOA class must be moved in package org.objectweb.corba.util.lib
import org.objectweb.ccm.CORBA.TheRootPOA;

/**
 * Abstract base class providing the main structure of
 * a CORBA server application.
 *
 * Subclasses must defined the before_run() and after_run() methods.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class ApplicationServerBase
              extends ApplicationBase
           implements org.objectweb.corba.command.api.ApplicationServer
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ApplicationServerBase()
    {
        // Calls the ApplicationBase constructor.
        super();
    }

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     */
    public
    ApplicationServerBase(CommandLine commandLine)
    {
        // Calls the ApplicationBase constructor.
        super(commandLine);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application, i.e.
     * 1 - Initializes the Root POA,
     * 2 - Actives it,
     * 3 - Calls the before_run() method,
     * 4 - Runs the ORB,
     * 5 - Calls the after_run() method.
     *
     * @param args The command line arguments.
     *
     * @return The status returned by the after_run() method or zero.
     */
    public int
    run(String[] args)
    {
        // Initializes the Root POA.
        org.omg.PortableServer.POA rootPOA = TheRootPOA.getRootPOA();

        // Activates the Root POA.
        try
        {
            rootPOA.the_POAManager().activate();
        }
        catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive exc)
        {
            // Wrap the PortableServer::POAManager::AdapterInactive exception.
            throw new org.objectweb.corba.util.api.UserExceptionWrapper(exc);
        }

        // Before running the ORB.
        int status = before_run(args);

        if(status != 0)
        {
            return status;
        }

        // Runs the ORB.
        TheORB.run();

        getConsole().message("Shutdown completed.");

        try
        {
            // After running the ORB.
            after_run();
        }
        catch(Exception exc)
        {
        }

        return 0;
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.command.api.ApplicationServer
    //
    // ==================================================================

    /**
     * Before running the CORBA::ORB object.
     *
     * Must be defined in subclasses.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public abstract int
    before_run(String[] args);

    /**
     * After running the CORBA::ORB object.
     *
     * Must be defined in subclasses.
     */
    public abstract void
    after_run();
}
