/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey, Philippe Merle.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
// import org.objectweb.corba.command.api.PSDLtoPSDL;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.corba.ast.api.FileScope;

/**
 * Implementation of the psdl_psdl command.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.2
 */

public class PSDLtoPSDL
     extends CompilerGeneratorBase
  implements org.objectweb.corba.command.api.PSDLtoPSDL
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    PSDLtoPSDL()
    {
        // Calls the CompilerGeneratorBase constructor.
        super(new DefaultCommandLine("psdl",
                                     "file",
                                     "Compile a file and regenerate the OMG PSDL part only",
                                     true),
              "generated.psdl");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        String filename = args[0];

        // Inits the OpenCCM's Abstract Syntax Tree.
        if(!initAST())
            return -1;

        // Compiles the file.
        FileScope fileScope = compile(filename);
        if(fileScope == null)
            return -1;

        // Generates PSDL (inputfile, outputfile)
        if (!generate_psdl(fileScope, filename, getOutputFileName()))
            return -1;

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Generator
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.command.api.PSDLtoPSDL
    //
    // ==================================================================

    /**
     * Generates PSDL from an AST FileScope instance.
     *
     * Note that checkComponentRepository() and initAST() methods
     * must be called before.
     *
     * @param filescope The source AST FileScope returned by the compile() method.
     * @param inputfile The source file name.
     * @param outputfile The target file name.
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_psdl(org.objectweb.corba.ast.api.FileScope filescope,
                  String inputfile,
                  String outputfile)
    {
        // Launchs the generator.
        org.objectweb.corba.generator.psdl.api.PSDLGenerator gen =
                         new org.objectweb.corba.generator.psdl.lib.PSDLGenerator(getAST());
        gen.initialize(outputfile, "PSDLtoPSDL");

        // Generates.
        getConsole().message("Generating PSDL from " + inputfile + "...");
        try
        {
            gen.generate( filescope, org.objectweb.corba.ast.api.DeclarationKind.dk_psdl );
        }
        catch(org.objectweb.corba.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }

        getConsole().message("PSDL generated in " + outputfile + ".");
        return true;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        PSDLtoPSDL psdl2psdl = new PSDLtoPSDL();
        psdl2psdl.runMain(args);
    }
}
