/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.generator.java.lib;

/** To access all the AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access AST TypeKind. */
import org.objectweb.corba.ast.api.TypeKind;

/** To use GenerationException. */
import org.objectweb.corba.generator.common.lib.GenerationException;



/**
 * Utility class for Java mapping.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.4
 */

public class JavaTranslator
  implements org.objectweb.corba.generator.java.api.JavaTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Table with declaration absolute names translated into java
     **/
    protected org.objectweb.ccm.util.Table abs_names_;

    /**
     * Java keywords table.
     **/
     protected org.objectweb.ccm.util.Table keywords_;


    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    JavaTranslator()
    {
        abs_names_ = new org.objectweb.ccm.util.Table();
        keywords_ = new org.objectweb.ccm.util.Table();
        keywords_.put("public", new Object());
        keywords_.put("protected", new Object());
        keywords_.put("private", new Object());
        keywords_.put("class", new Object());
        keywords_.put("interface", new Object());
        keywords_.put("abstract", new Object());
        keywords_.put("boolean", new Object());
        keywords_.put("break", new Object());
        keywords_.put("byte", new Object());
        keywords_.put("case", new Object());
        keywords_.put("catch", new Object());
        keywords_.put("char", new Object());
        keywords_.put("const", new Object());
        keywords_.put("continue", new Object());
        keywords_.put("default", new Object());
        keywords_.put("do", new Object());
        keywords_.put("double", new Object());
        keywords_.put("else", new Object());
        keywords_.put("extends", new Object());
        keywords_.put("final", new Object());
        keywords_.put("finally", new Object());
        keywords_.put("float", new Object());
        keywords_.put("for", new Object());
        keywords_.put("goto", new Object());
        keywords_.put("if", new Object());
        keywords_.put("implements", new Object());
        keywords_.put("import", new Object());
        keywords_.put("instanceof", new Object());
        keywords_.put("int", new Object());
        keywords_.put("long", new Object());
        keywords_.put("native", new Object());
        keywords_.put("new", new Object());
        keywords_.put("package", new Object());
        keywords_.put("return", new Object());
        keywords_.put("short", new Object());
        keywords_.put("static", new Object());
        keywords_.put("super", new Object());
        keywords_.put("switch", new Object());
        keywords_.put("synchronized", new Object());
        keywords_.put("this", new Object());
        keywords_.put("throw", new Object());
        keywords_.put("throws", new Object());
        keywords_.put("transient", new Object());
        keywords_.put("try", new Object());
        keywords_.put("void", new Object());
        keywords_.put("volatile", new Object());
        keywords_.put("while", new Object());
        keywords_.put("true", new Object());
        keywords_.put("false", new Object());
        keywords_.put("null", new Object());
        keywords_.put("clone", new Object());
        keywords_.put("equals", new Object());
        keywords_.put("finalize", new Object());
        keywords_.put("getClass", new Object());
        keywords_.put("hashCode", new Object());
        keywords_.put("notify", new Object());
        keywords_.put("notifyAll", new Object());
        keywords_.put("toString", new Object());
        keywords_.put("wait", new Object());
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Translate a TypeRef into its java mapping.
     *
     * @param typeref - The TypeRef to translate.
     *
     * @return The resulting mapping.
     **/
    public java.lang.String
    toJava(org.objectweb.corba.ast.api.TypeRef typeref)
    {
        TypeKind tk = typeref.getTypeKind();

        if (tk == TypeKind.tk_null)
            return "null";
        else if (tk == TypeKind.tk_void)
            return "void";
        else if (tk == TypeKind.tk_short)
            return "short";
        else if (tk == TypeKind.tk_ushort)
            return "short";
        else if (tk == TypeKind.tk_long)
            return "int";
        else if (tk == TypeKind.tk_ulong)
            return "int";
        else if (tk == TypeKind.tk_longlong)
            return "long";
        else if (tk == TypeKind.tk_ulonglong)
            return "long";
        else if (tk == TypeKind.tk_float)
            return "float";
        else if (tk == TypeKind.tk_double)
            return "double";
        else if (tk == TypeKind.tk_longdouble)
            return "double";
        else if (tk == TypeKind.tk_boolean)
            return "boolean";
        else if (tk == TypeKind.tk_char)
            return "char";
        else if (tk == TypeKind.tk_wchar)
            return "char";
        else if (tk == TypeKind.tk_octet)
            return "byte";
        else if (tk == TypeKind.tk_any)
            return "org.omg.CORBA.Any";
        else if (tk == TypeKind.tk_Principal)
            return "org.omg.CORBA.Principal";
        else if (tk == TypeKind.tk_TypeCode)
            return "org.omg.CORBA.TypeCode";
        else if (tk == TypeKind.tk_objref)
            return "org.omg.CORBA.Object";
        else if (tk == TypeKind.tk_string)
            return "String";
        else if (tk == TypeKind.tk_wstring)
            return "String";
        else if (tk == TypeKind.tk_fixed)
            return "java.Math.BigDecimal";

        else if ( tk == TypeKind.tk_struct ||
                  tk == TypeKind.tk_union ||
                  tk == TypeKind.tk_enum ||
                  tk == TypeKind.tk_exception ||
                  tk == TypeKind.tk_value_box ||
                  tk == TypeKind.tk_value ||
                  tk == TypeKind.tk_native ||
                  tk == TypeKind.tk_abstract_interface ||
                  tk == TypeKind.tk_local_interface ||
                  tk == TypeKind.tk_interface ||
                  tk == TypeKind.tk_component ||
                  tk == TypeKind.tk_home ||
                  tk == TypeKind.tk_event )
            {
                org.objectweb.corba.ast.api.Declaration decl;
                decl = (org.objectweb.corba.ast.api.Declaration)typeref;
                return getAbsoluteName(decl);
            }

        else if (tk == TypeKind.tk_alias)
        {
            org.objectweb.corba.ast.api.AliasDecl alias;
            alias  = (org.objectweb.corba.ast.api.AliasDecl)typeref;
            return toJava(alias.getType());
        }

        else if ( tk == TypeKind.tk_sequence ||
                  tk == TypeKind.tk_array )
        {
            org.objectweb.corba.ast.api.TypeRefWithContent content;
            content = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
            java.lang.String str = toJava(content.getContentType());
            return str+"[]";
        }
        else
            return "";
    }

    /**
     * Translate a PsdlTypeRef into its java mapping.
     *
     * @param type - The PsdlTypeRef to translate.
     *
     * @return The resulting mapping.
     **/
    public java.lang.String
    toJava(org.objectweb.corba.ast.api.PsdlTypeRef type)
    {
        org.objectweb.corba.ast.api.TypeRef idl_type = null;
        org.objectweb.corba.ast.api.StorageTypeBase st_base = null;

        if (type == null)
        {
            return "";
        }
        idl_type = type.getTypeRef();
        if (idl_type != null)
        {
            return toJava(idl_type);
        }
        else
        {   // type is a psdl type
            st_base = type.getStorageTypeBase();
            return getAbsoluteName(st_base);
        }
    }

    /**
     * Translate a parameter into its java mapping.
     *
     * @param typeref - The TypeRef to translate.
     * @param mode - The parameter mode (in, out or inout).
     *
     * @return The resulting mapping.
     **/
    public java.lang.String
    toJava(org.objectweb.corba.ast.api.TypeRef type,
           int mode)
    {
        if (mode==org.omg.CORBA.ParameterMode._PARAM_IN)
            return toJava(type);

        // mode is 'out' or 'inout'
        TypeKind tk = type.getTypeKind();
        if ( tk == TypeKind.tk_null ||
             tk == TypeKind.tk_void )
                throw new Error("IDL null and void types could not be used for out or inout parameters!!!");

        else if (tk == TypeKind.tk_short )
            return "org.omg.CORBA.ShortHolder";
        else if (tk == TypeKind.tk_ushort )
            return "org.omg.CORBA.ShortHolder";
        else if (tk == TypeKind.tk_long )
            return "org.omg.CORBA.IntHolder";
        else if (tk == TypeKind.tk_ulong )
            return "org.omg.CORBA.IntHolder";
        else if (tk == TypeKind.tk_longlong )
            return "org.omg.CORBA.LongHolder";
        else if (tk == TypeKind.tk_ulonglong )
            return "org.omg.CORBA.LongHolder";
        else if (tk == TypeKind.tk_float )
            return "org.omg.CORBA.FloatHolder";
        else if (tk == TypeKind.tk_double )
            return "org.omg.CORBA.DoubleHolder";
        else if (tk == TypeKind.tk_longdouble )
            throw new Error("Java mapping for IDL longdouble is unspecified!!!");
        else if (tk == TypeKind.tk_boolean )
            return "org.omg.CORBA.BooleanHolder";
        else if (tk == TypeKind.tk_char )
            return "org.omg.CORBA.CharHolder";
        else if (tk == TypeKind.tk_wchar )
            return "org.omg.CORBA.CharHolder";
        else if (tk == TypeKind.tk_octet )
            return "org.omg.CORBA.ByteHolder";
        else if (tk == TypeKind.tk_any )
            return "org.omg.CORBA.AnyHolder";
        else if (tk == TypeKind.tk_Principal )
            return "org.omg.CORBA.PrincipalHolder";
        else if (tk == TypeKind.tk_TypeCode )
            return "org.omg.CORBA.TypeCodeHolder";
        else if (tk == TypeKind.tk_objref )
            return "org.omg.CORBA.ObjectHolder";
        else if (tk == TypeKind.tk_string )
            return "org.omg.CORBA.StringHolder";
        else if (tk == TypeKind.tk_wstring )
            return "org.omg.CORBA.StringHolder";
        else if (tk == TypeKind.tk_fixed )
            return "org.omg.CORBA.FixedHolder";
        else if (tk == TypeKind.tk_alias )
            {
                org.objectweb.corba.ast.api.TypeRef ct;
                ct = ((org.objectweb.corba.ast.api.AliasDecl)type).getType();
                if ((ct.getTypeKind()==TypeKind.tk_array) ||
                    (ct.getTypeKind()==TypeKind.tk_sequence))
                    return getAbsoluteName((org.objectweb.corba.ast.api.AliasDecl)type) + "Holder";
                else
                    return toJava(ct, mode);
            }
        else if ( tk == TypeKind.tk_sequence ||
                  tk == TypeKind.tk_array )
            throw new Error("Not implemented!!!");
        else if ( tk == TypeKind.tk_struct ||
                  tk == TypeKind.tk_union ||
                  tk == TypeKind.tk_enum ||
                  tk == TypeKind.tk_exception ||
                  tk == TypeKind.tk_value_box ||
                  tk == TypeKind.tk_value ||
                  tk == TypeKind.tk_native ||
                  tk == TypeKind.tk_abstract_interface ||
                  tk == TypeKind.tk_local_interface ||
                  tk == TypeKind.tk_interface ||
                  tk == TypeKind.tk_component ||
                  tk == TypeKind.tk_home ||
                  tk == TypeKind.tk_event )
            return getAbsoluteName((org.objectweb.corba.ast.api.Declaration)type) + "Holder";
        else
            throw new Error("Other cases are not supported!!!");
    }

    /**
     * Translate a psdl parameter into its java mapping.
     *
     * @param type - The PsdlTypeRef to translate.
     * @param mode - The parameter mode (in, out or inout).
     *
     * @return The resulting mapping.
     **/
    public java.lang.String
    toJava(org.objectweb.corba.ast.api.PsdlTypeRef type,
           int mode)
    {
        org.objectweb.corba.ast.api.TypeRef idl_type = null;
        org.objectweb.corba.ast.api.StorageTypeBase st_base = null;

        if (type == null)
        {
            return "";
        }
        idl_type = type.getTypeRef();
        if (idl_type != null)
        {
            return toJava(idl_type, mode);
        }
        else
        {   // type is a psdl type
            st_base = type.getStorageTypeBase();
            return getAbsoluteName(st_base)+"Holder";
        }
    }

    /**
     * Check if id is a java keyword.
     *
     * @param id - The string to check
     *
     * @return A valid string.
     **/
    public java.lang.String
    checkKeywords(java.lang.String id)
    {

        if (keywords_.containsKey(id))
           return "_"+id;

        return id;
    }

    /**
     * Get the java absolute name of a declaration
     * ( i.e. package.decl_name ).
     *
     * @param decl - The declaration to translate
     *
     * @return The java absolute name.
     **/
    public java.lang.String
    getAbsoluteName(Declaration decl)
    {
        if (decl==null)
            return "";

        // check if the absolute name has already been generated.
        java.lang.String res = null;
        res = (java.lang.String)abs_names_.get(decl.getId());
        if (res!=null)
            return res;

        // translate the IDL prefix into a java prefix.
        res = "";
        java.lang.String prefix = decl.getPrefix();
        int idx, idx2;
        if (!prefix.equals(""))
        {
            idx = prefix.indexOf('.');
            idx2 = 0;
            while (idx!=-1)
            {
                res = prefix.substring(idx2, idx+1) + res;
                idx2 = idx+1;
                idx = prefix.indexOf('.', idx2);
            }
            res = prefix.substring(idx2) + '.' + res;
        }

        // translate the IDL scoped name into a Java absolute name
        java.lang.String abs_name = decl.getAbsoluteName();
        idx = abs_name.indexOf(':', 2);
        idx2 = 2;
        while (idx!=-1)
        {
            res = res + abs_name.substring(idx2, idx) + '.';
            idx2 = idx+2;
            idx = abs_name.indexOf(':', idx2);
        }
        res = res + abs_name.substring(idx2);

        // check special cases ie nested types within interfaces
        org.objectweb.corba.ast.api.Scope parent = decl.getParent();
        long parent_kind = parent.getDeclKind();
        long decl_kind = decl.getDeclKind();
        if (decl_kind==DeclarationKind.dk_constant)
        {
            if ((parent_kind!=DeclarationKind.dk_interface) &&
                (parent_kind!=DeclarationKind.dk_abstract_interface) &&
                (parent_kind!=DeclarationKind.dk_local_interface))
                res = res + ".value";
        }
        else if ((decl_kind==DeclarationKind.dk_exception) ||
                 (decl_kind==DeclarationKind.dk_alias) ||
                 (decl_kind==DeclarationKind.dk_struct) ||
                 (decl_kind==DeclarationKind.dk_union) ||
                 (decl_kind==DeclarationKind.dk_enum) ||
                 (decl_kind==DeclarationKind.dk_native))
        {
            if ((parent_kind==DeclarationKind.dk_interface) ||
                (parent_kind==DeclarationKind.dk_abstract_interface) ||
                (parent_kind==DeclarationKind.dk_local_interface) ||
                (parent_kind==DeclarationKind.dk_home) ||
                (parent_kind==DeclarationKind.dk_value) ||
                (parent_kind==DeclarationKind.dk_event) ||
                (parent_kind==DeclarationKind.dk_component))
            {
                idx = res.lastIndexOf('.');
                res = res.substring(0, idx) + "Package" + res.substring(idx);
            }
        }

        // store the computed absolute name
        abs_names_.put(decl.getId(), res);
        return res;
    }

    /**
     * Get the package of a declaration.
     *
     * @param decl - The declaration to translate
     *
     * @return The java package name.
     **/
    public java.lang.String
    getPackage(Declaration decl)
    {
        java.lang.String abs_name = getAbsoluteName(decl);
        int idx = abs_name.lastIndexOf('.');
        if (idx==-1)
            return "";

        return abs_name.substring(0, idx);
    }

    /**
     * Get the java path of a declaration.
     * This don't include the declaration's java name.
     * The package name is translated to a path.
     *
     * @param decl - The declaration to translate
     *
     * @return The java package name.
     **/
    public String
    getAsDirectory(String package_name)
    {
        if (package_name == null)
            return "./";

        return package_name.replace('.', java.io.File.separatorChar);
    }

    /**
     * Get the java path of a declaration.
     * This don't include the declaration's java name.
     * The package name is translated to a path.
     *
     * @param decl - The declaration to translate
     *
     * @return The java package name.
     **/
    public java.lang.String
    getAsDirectory(Declaration decl)
    {
        java.lang.String abs_name = getAbsoluteName(decl);
        int idx = abs_name.indexOf('.');
        if (idx==-1)
            return "./";

        java.lang.String dir = "";
        int idx2 = 0;
        while (idx!=-1)
        {
            dir = dir + abs_name.substring(idx2, idx) + '/';
            idx2 = idx+1;
            idx = abs_name.indexOf('.', idx2);
        }

        return dir;
    }

    /**
     * Create directories for a given path,
     * including any necessary but nonexistent parent directories.
     *
     * @param dir - A directory.
     **/
    public void
    mkdir(java.io.File dir)
    throws GenerationException
    {
        if (dir.exists())
        {
            if (!dir.isDirectory())
            {
                String msg = new String(dir + " is not a directory!");
                throw new GenerationException(msg);
            }
        }
        else
        {
            dir.mkdirs();
        }
    }

    /**
     * Create directories for a given path,
     * including any necessary but nonexistent parent directories.
     *
     * @param dir_name - A pathname string.
     *
     * @return The corresponding File.
     **/
    public java.io.File
    mkdir(String dir_name)
    throws GenerationException
    {
        java.io.File dir = new java.io.File(dir_name);
        mkdir(dir);
        return dir;
    }
}
