@echo off
rem # ====================================================================
rem #
rem # OpenCCM: The Open CORBA Component Model Platform
rem # Copyright (C) 2000-2002 USTL - LIFL - GOAL
rem # Contact: openccm-team@objectweb.org
rem #
rem # This library is free software; you can redistribute it and/or
rem # modify it under the terms of the GNU Lesser General Public
rem # License as published by the Free Software Foundation; either
rem # version 2.1 of the License, or any later version.
rem #
rem # This library is distributed in the hope that it will be useful,
rem # but WITHOUT ANY WARRANTY; without even the implied warranty of
rem # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
rem # Lesser General Public License for more details.
rem #
rem # You should have received a copy of the GNU Lesser General Public
rem # License along with this library; if not, write to the Free Software
rem # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
rem # USA
rem #
rem # Initial developer(s): Christophe Contreras
rem # Contributor(s): Christophe Demarey, Jerome Offroy.
rem #
rem # ====================================================================
rem #
rem # Deploy an application with a *.cad file
rem #
rem # ====================================================================


rem #  =====================
rem #  Compute Command Line
rem #  =====================

set /a nb_args=0
for %%i in (%*) do set /a nb_args=nb_args+1

if %nb_args% lss 1 goto help

set with_ots="false"
set with_trace="false"
set services_list=
set /a nb=0
set command_args=

:initial
if "%1" == "-h" goto help
if "%1" == "--ots" goto ots
if "%1" == "--trace" goto trace
if %nb% geq %nb_args% (goto end_command_line) else (goto arguments)

:ots
set with_ots="true"
set services_list=%services_list% Transactional
shift
set /a nb=nb+1
goto initial

:trace
set with_trace="true"
set services_list=%services_list% Trace
shift
set /a nb=nb+1
goto initial

:arguments
set command_args=%command_args% %1
shift
set /a nb=nb+1
goto initial

:help
echo usage: ccm_deploy [--ots] [--trace] ^<file.cad^>
goto :eof

:end_command_line


rem #  ============================
rem #  OpenCCM Platform installed ?
rem #  ============================

:platform_installed
call %OpenCCM_HOMEDIR%\bin\ccm_installed
if %CCM_INSTALLED%==0 goto :eof

rem #  ======================
rem #  Name Service started ?
rem #  ======================

rem # Obtain the IOR of the started Name Service.
call ns_ior > %OpenCCM_CONFIG_DIR%\ComponentServers\ccm_deploy.tmp
for /f %%i in (%OpenCCM_CONFIG_DIR%\ComponentServers\ccm_deploy.tmp) do set ior_NS=%%i
del %OpenCCM_CONFIG_DIR%\ComponentServers\ccm_deploy.tmp

rem # Check if a Name Service is started.
if defined ior_NS goto ior_ots

echo Error: The Name Service is not started!
goto :eof

rem #  ===============
rem #  Obtain IOR OTS.
rem #  ===============

:ior_ots

if %with_ots% == "false" goto conf

rem # Obtain the IOR of the started Transaction Service.
call ots_ior > %OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp
for /f %%i in (%OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp) do set ior_OTS=%%i
del %OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp

rem # Check if a Transaction Service is started.
if defined ior_OTS goto conf

echo Error: The Transaction Service is not started!
goto :eof


rem #  ==============
rem #  Run deployment
rem #  ==============

:conf
set jvm_args=
set run_id=deploy

if %with_ots% == "false" goto trace_service
rem # Transaction Service
rem #
set jvm_args=-Dior_ots=%ior_OTS%
set run_id=%run_id%_ots

:trace_service
if %with_ots% == "false" goto run
set run_id=%run_id%_trace

:run
set jvm_args=%jvm_args% -Dior_ns=%ior_NS%
set args=--runid %run_id% %command_args% 

launcher exec "%jvm_args%" "%args% %OpenCCM_HOMEDIR%/xml/launcher/Deployment.xml"



